<?php
    //error_reporting(E_ALL);
    session_start();

    // ====== Variables ======
    $root = "..";
    $users_folder = "_users";
    $groups_path = "$users_folder/.groups";
    $localization_folder = "localization";
    $login_error = false;
    $login_disabled = false;

    $security_log = "$users_folder/security_log.csv";
    $security_locks = "$users_folder/security_lock.csv";

    require("modules/_update_version.php");

    // ====== Load Functions Libraries ======

    require "repository/functions.php";
    require "repository/cms.php";
    require "repository/xml.php";

    $site_options = loadIni("xable.ini", "options");
    $settings = loadXml("$root/settings.json");

    $ini_pathes = loadIni("xable.ini", "pathes");
    $pass_path = $ini_pathes["passwords"];

    if(!isset($site_options["security"])) {
        $site_options["security"] = "log_length:50,ban_time:60,login_errors:3,ip_errors:10"; // Default
    }
    $_SESSION["security_options"] = stringToObject($site_options["security"]);
    //arrayList($_SESSION["security_options"]);

    // Get Back URL if any
    if(isset($_GET["back_url"]) && $_GET["back_url"] != "") {
        $_SESSION["login_url"] = $_GET["back_url"];
    };

    // ===================================================
    //                    LOCALIZATION
    // ===================================================

    // ====== LANGUAGES ======
    // Set admin master language
    if(is_string($site_options['admin_lang'])) {
        $_SESSION['admin_lang'] = $site_options['admin_lang'];
    }
    else {
        $_SESSION['admin_lang'] = "pl"; // default
    }

    loadLocalization($localization_folder, $_SESSION['admin_lang']);

    // ===================================================
    //                      FUNCTIONS
    // ===================================================

    function goToPanel() {
        if(isset($_SESSION["login_url"]) && $_SESSION["login_url"] != "") {
            $location = array_shift(explode("&popup=", $_SESSION["login_url"]));
            unset($_SESSION["login_url"]);
            header("Location: ".$location);
        }
        else {
            header("Location: index.php");
        }
        exit();
    };

    function getAdminFolder() {
        $admin_folder = false;
        $url = $_SERVER['REQUEST_URI'];
        if(substr($url, strlen($url) - 1) == "/") { $url = substr($url, 0, strlen($url) - 1); };
        if(!is_string(path($url, "extension")) || path($url, "extension") == "") {
            $admin_folder = array_pop(explode("/", $url));
        }
        else {
            $admin_folder = array_pop(explode("/", path($url, "dirname")));
        };
        return $admin_folder;
    };

    function loadUsers($pass_path) {
        $users = array();
        $file = file($pass_path);
        foreach($file as $user) {
            
            $user = explode(":", $user);
            if(count($user) == 2) {
                $user = array_map("trim", $user);
                $users[ $user[0] ] = $user[1];
            };
        };
        return $users;
    };

    function loadUserGroup($user, $groups_path) {
        $user_group = false;
        $file = file($groups_path);
        foreach($file as $group) {
            $group = explode(":", $group);
            $group = array_map("trim", $group);
            $group_name = $group[0];
            $group_users = array_map("trim", explode(" ", $group[1]));
            if(in_array($user, $group_users)) { $user_group = $group_name; };
        };
        return $user_group;
    };

    function postInstall($root) {
        $install_folder = "$root/install";
        if(file_exists("$install_folder/xable")) {
            removeDir($install_folder);
            $_SESSION['popup'] = localize("xable-welcome-info")."|done";
            $_SESSION["published_flag"] = true;
            addLog("INSTALL", "Xable-CMS");
        };
    };

    function checkIp($security_locks) {
        $ip = userIP();
        //$ban_time = 3600; // seconds
        $ban_time = $_SESSION["security_options"]["ban_time"] * 60; // ban_time, converted from minutes to seconds
        //echo "! $ip<br>\n";
        $time = date("Y-m-d G:i:s");
        $timestamp = strtotime($time);
        $error_info = false;
        $security_list = explode("\n", file_get_contents($security_locks));
        $security_list_headers = array_shift($security_list);
        $security_length = count($security_list);
        foreach($security_list as $log) {
            //echo "> $log<br>\n";
            $log = explode(";", trim($log)); // [0]: time, [1]: ip, [2]: user, [3]: status
            if($log[1] == $ip) {
                $time_diff = ($timestamp - strtotime($log[0])); // seconds old
                if($time_diff < $ban_time) {
                    //echo "> > Disabled: ".$log[3]."<br>\n";
                    $expiration = date("Y-m-d G:i:s", (strtotime($log[0]) + $ban_time));
                    $error_info = "disabled-login-ip;".$expiration;
                }
                else {
                    //echo "> > Obsolete input: ".$log[0]."<br>\n";
                }
            }
        }
        return $error_info;
    }

    function loginLog($security_locks, $security_log, $user, $status) {
        // $status = <string> login-succes, wrong-login, wrong-password
        //echo "loginLog($security_locks, $security_log, $user, $status)<br>\n";
        $ip = userIP();
        $time = date("Y-m-d G:i:s");
        $timestamp = strtotime($time);
        $log_max = $_SESSION["security_options"]["log_length"]; // Number of log items keeped in log file
        $errors_max_login = $_SESSION["security_options"]["login_errors"]; // Number of max wrong password attempts per login
        $errors_max_ip = $_SESSION["security_options"]["ip_errors"]; // Number of max wrong password attempts per IP
        $ban_time = $_SESSION["security_options"]["ban_time"] * 60; // ban_time, converted from minutes to seconds
        $errors = [ "ip" => 0, "login" => 0 ];
        $error_info = false;

        $security_list = explode("\n", file_get_contents($security_locks));
        $security_list_headers = array_shift($security_list);
        $security_length = count($security_list);

        foreach($security_list as $log) {
            $log = explode(";", trim($log)); // [0]: time, [1]: ip, [2]: user, [3]: status
            if(!$error_info && ($log[1] == $ip || $log[2] == $user)) {
                $time_diff = ($timestamp - strtotime($log[0])); // seconds old
                if($time_diff < $ban_time) {
                    //echo "> ".$log[0]." -> ".$time." = $time_diff<br>\n";
                    $expiration = date("Y-m-d G:i:s", (strtotime($log[0]) + $ban_time));
                    if($log[1] == $ip) { $error_info = "disabled-login-ip;".$expiration; }
                    else { $error_info = "disabled-login-login;".$expiration;}
                    //echo "ERROR: $error_info<br>\n";
                }
                else {
                    //echo "> Obsolete input: ".$log[0]."<br>\n";
                }
            }
        }
        
        if($error_info == false) {
            $log_list = explode("\n", file_get_contents($security_log));
            $log_list_headers = array_shift($log_list);
            //echo "ip: $ip, user: $user<br>\n";
            foreach($log_list as $log) {
                $log = explode(";", trim($log)); // [0]: time, [1]: ip, [2]: user, [3]: status
                if($log[1] == $ip || $log[2] == $user) {
                    //echo "[0]:".$log[0].", [1]:".$log[1].", [2]:".$log[2].", [3]:".$log[3]."<br>\n";
                    $time_diff = ($timestamp - strtotime($log[0])); // seconds old
                    //echo "age: $time_diff s<br>\n";
                    if($error_info == false && $time_diff < $ban_time) {
                        
                        if($log[3] == "login-success") {
                            //echo $log[0]." > login-success<br>\n";
                            $errors["ip"] = 0;
                            $errors["login"] = 0;
                        }
                        else {
                            //echo $log[0]." > ".$log[3]."<br>\n";
                            if($log[1] == $ip) {
                                $errors["ip"] = $errors["ip"] + 1;
                                if($errors["ip"] >= ($errors_max_ip - 1)) {
                                    // Add disabled IP
                                    $disable = [ $time, $ip, "", $status ];
                                    $security_list[] = join(";", $disable);
                                    $expiration = date("Y-m-d, G:i:s", (strtotime($log[0]) + $ban_time));
                                    $error_info = "disabled-login-ip;".$expiration;
                                }
                            }
                            if($log[2] == $user) {
                                $errors["login"] = $errors["login"] + 1;
                                //echo "- count: ".$errors["login"]."<br>\n";
                                if($errors["login"] >= ($errors_max_login - 1)) {
                                    // Add disabled login
                                    $disable = [ $time, "", $user, $status ];
                                    $security_list[] = join(";", $disable);
                                    $expiration = date("Y-m-d, G:i:s", (strtotime($log[0]) + $ban_time));
                                    $error_info = "disabled-login-login;".$expiration;
                                }
                            }
                        }
                    }
                }
            }
            
            $log = [ $time, userIP(), $user, $status ]; // [0]: time, [1]: ip, [2]: user, [3]: status
            $log_list[] = join(";", $log);

            $log_list = array_slice($log_list, -$log_max);
            //arrayList($log_list);
            safeSave($security_log, $log_list_headers."\n".join("\n", $log_list));

            if(count($security_list) != $security_length) {
                $security_list = array_slice($security_list, -$log_max);
                safeSave($security_locks, $security_list_headers."\n".join("\n", $security_list));
            }
            
        }
        return $error_info;
    };

    // ==========================================
    //          CHECK LOGIN / POST DATA
    // ==========================================

    // Already loaded
    if(
        is_string($_SESSION['logged_user']) &&
        $_SESSION['logged_user'] != "" &&
        is_string($_SESSION['ini_file']) &&
        $_SESSION['ini_file'] != "" &&
        $_SESSION['cms_url'] == getCmsUrl() &&
        $_SESSION['logged_ip'] == $_SERVER['REMOTE_ADDR']
    ) {
        //arrayList($_SESSION);
        goToPanel();
    }
    // Login & pass
    else if(is_string($login = $_POST['login']) && is_string($pass = $_POST['password'])) {
        $users = loadUsers($pass_path);

        // ====== Wrong login ======
        if(!isset($users[$login])) {
            $_SESSION['logged_user'] = false;
            $login_error = true;
            $login_disabled = loginLog($security_locks, $security_log, $login, "wrong-login");
        }
        // ====== Wrong password ======
        elseif($users[$login] != crypt($pass, "mn")) {
            $_SESSION['logged_user'] = false;
            $login_error = true;
            $login_disabled = loginLog($security_locks, $security_log, $login, "wrong-password");
        }
        // ====== Correct login & password ======
        else {
            $login_disabled = loginLog($security_locks, $security_log, $login, "login-success");
            if($login_disabled) {
                $login_error = true;
            }
            else {
                $group = loadUserGroup($login, $groups_path);
                if(!is_string($group)) { $group = "*none*"; };
                // Set session variables
                $_SESSION['logged_user'] = $login;
                $_SESSION['logged_group'] = $group;
                $_SESSION['logged_ip'] = $_SERVER['REMOTE_ADDR'];
                $_SESSION['ini_file'] = "$users_folder/$group.ini";
                $_SESSION['password_file'] = $pass_path;
                $_SESSION['groups_file'] = $groups_path;
                $_SESSION['admin_folder'] = getAdminFolder();
                $_SESSION['cms_url'] = getCmsUrl();
                // Redirect to panel
                goToPanel();
            }
        };
    }
    // Initial
    else {
        $login_disabled = checkIp($security_locks);
        if($login_disabled) {
            $login_error = true;
        }
    }




?>

<!doctype html>
<html lang='<?php echo $_SESSION["admin_lang"]; ?>'>
	<head>
        <?php
            $xable_date = array_pop(explode(";", $xable_version));
            echo "\n".
                "\t\t<!-- ======================================\n".
                "\t\t               ><.able CMS"."\n".
                "\t\t      (c)".substr($xable_date, 0, 4)." maciej@maciejnowak.com"."\n".
                "\t\t         v.".str_replace(";", ", build.", $xable_version)."\n".
                "\t\t====================================== -->\n";
        ?>
        
        <!-- Loader Style -->
        <style><?php include "style/loader.css"; ?></style>
        
        <!-- Loader Style / end -->
		<meta charset="UTF-8">
		<title><?php echo localize("xable-login"); ?></title>
        <meta name="viewport" content="width=device-width, initial-scale=1, maximum-scale=1, user-scalable=0">
        
		<link rel="stylesheet" type="text/css" href="style/loader.css" />
		<link rel="stylesheet" type="text/css" href="style/index.css<?php echo "?v=".$last_update; ?>" />
		<link rel="stylesheet" type="text/css" href="style/cms.css<?php echo "?v=".$last_update; ?>" />
        <link rel="stylesheet" type="text/css" href="style/colors.css<?php echo "?v=".$last_update; ?>" />
        <link rel="stylesheet" type="text/css" href="style/login.css<?php echo "?v=".$last_update; ?>" />
        <link rel="stylesheet" type="text/css" href="style/_responsive.css<?php echo "?v=".$last_update; ?>" />
		<link rel="stylesheet" type="text/css" href="style/foundation-icons.css" />
		<link href='https://fonts.googleapis.com/css?family=Lato:100,300,400,700,900|Inconsolata:400,700|Audiowide&subset=latin,latin-ext' rel='stylesheet' type='text/css'>


	</head>

	<body>
        <div id='loader'>
            <div id="loadingProgressG">
                <div id="loadingProgressG_1" class="loadingProgressG"></div>
            </div>
        </div>

        <form id="password" method='post' action='login.php'>
            <div id="page_fader"></div>
            <div id="popup_container">
                <div id="popup_box">
                    <header>
                        <h1><strong>&gt;&lt;</strong>.able<span>CMS</span></h1>
                        <h2>&copy;<?php echo substr($xable_date, 0, 4); ?> by <a href='mailto:maciej@maciejnowak.com'>maciej@maciejnowak.com</a></h2>
                    </header>
                    <div id="form">
                        <div class='info'>
                            <?php
                                echo "\n";
                                if($login_disabled && $login_disabled != "") {
                                    list($status, $expire) = explode(";", $login_disabled);
                                    $info = localize($status);
                                    $info = str_replace("@expire", $expire, $info);
                                    echo "<p class='error'>".$info."</p>\n";
                                }
                                elseif($login_error == true) {
                                    echo "<p class='error'>".localize("login-fail")."</p>\n";
                                }
                                elseif(is_string($_GET['info'])) {
                                    echo "<p>".$_GET['info']."</p>\n";
                                };
                            ?>
                        </div>
                        <div class='inputs'>
                            <div class='input_box'>
                                <input type='text' class='string' id='login' name='login' placeholder='<?php echo localize("login-label"); ?>' required>
                            </div>
                            <div class='input_box'>
                                <input type='password' class='string' id='password' name='password' placeholder='<?php echo localize("password-label"); ?>' value='' required>
                            </div>
                        </div>

                        <div class='buttons'>
                            <button class='confirm' type='submit'><?php echo localize("login-button"); ?></button>
                            <!-- <button class='cancel'><?php echo localize("cancel-label"); ?></button> -->
                        </div>
                        
                        <div class='links'>
                            <!-- Reset Password -->
                            <?php
                                // Reset password
                                echo "\n";
                                $mailer_email = readXml($settings, "mailer email");
                                if($site_options["reset_password"] == "true" && is_string($mailer_email) && count(explode("@", $mailer_email)) == 2) {
                                    echo "<p class='reset_password'>".localize("forgot-password")."</p>\n";
                                }
                            ?>
                        </div>
                    </div>
                </div>
            </div>
        </form>
        
        <?php 
            if(is_string($popup)) { echo "<input id='popup' value='$popup'>\n"; };
            if(is_string($redirect)) { echo "<input id='redirect' value='$redirect'>\n"; };
            postInstall($root);
            exportLocalization();
        ?>
        
        <?php require "modules/_jquery_lib.php"; ?>
        
        <script src='script/login.js'></script>
        
	</body>
</html>
