// ===================================================
//             X.able JS/jQuery FUNCTIONS
//             (C)maciej@maciejnowak.com
// ===================================================
// build: 20201027
// ===================================================

// ==========================================
//                  FIXES
// ==========================================

// as of 1.4.2 the mobile safari reports wrong values on offset()
// http://dev.jquery.com/ticket/6446
// remove once it's fixed
/*
if(/webkit.*mobile/i.test(navigator.userAgent)) {
    (function($) {
        $.fn.offsetOld = $.fn.offset;
        $.fn.offset = function() {
            var result = this.offsetOld();
            result.top -= window.scrollY;
            result.left -= window.scrollX;
            return result;
        };
    })(jQuery);
};
*/

// ==========================================
//             Arrays / Objects
// ==========================================

/*
Array.prototype.joinValues = function(char) {
// -------------------------------------------
// this = <array> file path
// char = <string> join character
// -------------------------------------------
// RETURNS: <string> Joined array values
// -------------------------------------------
    var input_array = this;
    return Object.keys(input_array).map(function(i){ return input_array[i]; }).join(char);
};
*/

function arrayIntersection(array_1, array_2) {
// --------------------------------
// array_1 = <array>
// array_2 = <array>
// --------------------------------
// RETURN: <array> intersection of input arrays or <false> if no intersection found
// --------------------------------
    intersection = [];
    // Search speed optimalisation -> shorter loop
    if(array_1.length < array_2.length) { shorter = array_1; longer = array_2; }
    else { shorter = array_2; longer = array_1; };
    // Find intersections
    for(i = 0; i < shorter.length; i++) {
        if(longer.indexOf(shorter[i]) > -1) {
            intersection[intersection.length] = shorter[i];
        };
    };
    // Output
    if(intersection.length > 0) { return intersection; } else { return false; };
};

function arrayToJson(array, title, depth) {
// -------------------------------------------
// array = <array> Input array
// title = <string> <string> Array/subarray title
// depth = <integer> Left indent depth
// -------------------------------------------
// RETURNS: <string> Input array in JSON format
// -------------------------------------------
    if(typeof(title) != "string") { title = false; }
    if(typeof(depth) != "number") { depth = 0; }
    var json = [];
    var tab = "";
    for(i = 0; i < depth; i++) { tab = tab + "\t"; }
    if(Array.isArray(array)) { var brackets = [ "[", "]" ]; }
    else { var brackets = [ "{", "}" ]; };
    if(title) { json[json.length] = tab + "\"" + title + "\": " + brackets[0]; }
    for(key in array) {
        var val = array[key];
        if(typeof(val) == "object") {
            json[json.length] = arrayToJson(val, key, (depth + 1));
        }
        else if(brackets[0] == "[") {
            json[json.length] = tab + "\t\"" + val + "\",";
        }
        else {
            json[json.length] = tab + "\t\"" + key + "\": \"" + val + "\",";
        }
    }
    var last = json.length - 1;
    json[last] = json[last].substr(0, (json[last].length - 1));
    if(title) { json[json.length] = tab + brackets[1]; }
    json = json.join("\n");
    if(depth == 0) { json = "{\n" + json + "\n}"; }
    else { json = json + ","; };
    return json;
};

function deleteValue(arr, val) {
// -------------------------------------------
// arr = <array> input ARRay
// val = <integer/string> value to delete
// -------------------------------------------
// RETURN: <array> Input array without specified value
// -------------------------------------------
    var output = [];
    for(i in arr) {
        if(arr[i] != val) { output[output.length] = arr[i]; }
    }
    return output;
};

function objectSort(obj) {
// -------------------------------------------
// obj = <object>
// -------------------------------------------
// RETURN: <object> Object sorted by keys (subobject too)
// -------------------------------------------
    var output = {};
    Object.keys(obj).sort().forEach(function(key) {
        var val = obj[key];
        if(typeof(val) == "object") { val = objectSort(val); }
        output[key] = val;
    });
    return output;
}

function objectToXml(data, depth) {
// -------------------------------------------
// data = <array> or <object> variable(s) data
// depth = <number> array Depth level (optional)
// -------------------------------------------
// RETURNS: <string> Array content converted to XML string form
// -------------------------------------------
    if(typeof depth != "number") { depth = 0; }
    var xml = "";
    var i = "";
    for(i in data) {
        var indent = "";
        for(n = 0; n < depth; n++) { indent = indent + "\t"; };

        if(typeof data[i] == "object" || typeof data[i] == "array") {
            xml = xml + indent + "<" + i + ">\n";
            xml = xml + objectToXml(data[i], depth + 1);
            xml = xml + indent + "</" + i + ">\n";
        }
        else {
            xml = xml + indent + "<" + i + ">" + variableToString(data[i]) + "</" + i + ">\n";
        }
    }
    return xml;
};

function variableToString(string) {
// -------------------------------------------
// string = <string> 
// -------------------------------------------
// RETURNS: <string> Input string with problematic character replaced
// -------------------------------------------
    string = string + ""; // number/boolean to string conversion
    string = string.replace("<", "&lt;").replace(">", "&gt;");
    string = string.replace(/\n|<br>/gi, "[br]")
    return string;
};

// ==========================================
//                 Cookies
// ==========================================

function setCookie(cnames, cvalue, exdays) {
// -------------------------------------------
// cnames = <string> Cookie Name(s): "cname" or "cname1,cname2,cname3"
// cvalue = <string> Cookie Value or <undefined> for Delete mode
// exdays = <number> Expirations Days
// -------------------------------------------
// SETS: Cookie or delete cookie if no value specified
// -------------------------------------------
    if(jQuery.type(cvalue) == "number") { cvalue = "" + cvalue; }
    if(jQuery.type(cvalue) != "string" || jQuery.type(exdays) != "number") {
        cvalue = "false";
        exdays = -1
    }
    var d = new Date();
    d.setTime(d.getTime() + (exdays*24*60*60*1000));
    var expires = "expires="+ d.toUTCString();
    var cname_list = cnames.split(",");
    for(i in cname_list) {
        cname = cname_list[i];
        document.cookie = cname + "=" + cvalue + ";" + expires + ";path=/";
    }
};

function readCookies() {
// -------------------------------------------
// RETURNS: <object> Cookies: { "key1": "value1", "key2": "value2" }
// -------------------------------------------
    var cookies = {};
    var cookies_list = document.cookie.split(";").map(function(e){return e.trim();});
    for(i in cookies_list) {
        data = cookies_list[i].split("=");
        cookies[ data[0] ] = data[1];
    }
    if(Object.keys(cookies).length > 0) {
        return cookies;
    }
    else {
        return false;
    }
};

// ==========================================
//                 Numbers
// ==========================================

function digits(int, num) { // used in: currentTime(format)
// -------------------------------------------
// int = <integer> INTeger
// num = <integer> digits NUMber
// -------------------------------------------
// RETURN: <string> Integer with specified digits length, with leading zero added id needed
// -------------------------------------------
    int = int + ""; // convert to string
    while(int.length < num) { int = "0" + int; };
    return int;
};

Number.prototype.toDigits = function(digits) { // used in: translate()
// -------------------------------------------
// this = <number> or <string>
// num = <integer> minimal digits number
// -------------------------------------------
// RETURN: <string> Integer with specified digits length, with leading zero added id needed
// -------------------------------------------
    val = this;
    val = val + "";
    while(val.length < digits) { val = "0" + val; };
    return val;
};

// ==========================================
//                Validations
// ==========================================

function validateEmail(email) { // used in: applyStyle()
// --------------------------------
// Check if string is valid email
// --------------------------------
// RETURN: <string> input email or <false> if check failed
// --------------------------------
    characterTest = email.replace(/[^a-z0-9|^@|^.|^!|^#|^$|^%|^&|^?|^'|^*|^_|^\-|^+|^=|^\||^~|^{|^}]/gi, "");
    emailParts = email.split("@");
    if(
        email == characterTest &&
        email.indexOf("..") < 0 &&
        emailParts.length == 2 &&
        emailParts[0] != "-" &&
        emailParts[0] != "_" &&
        emailParts[1].length > 2 &&
        emailParts[1].split(".").length >= 2 &&
        emailParts[1].substr(0, 1) != "." &&
        emailParts[1].substr(emailParts[1].length - 1) != "."
        
    ) {
        return email;
    }
    else {
        return false;
    };
};

function validateUrl(url) { // used in: applyStyle()
// --------------------------------
// Check if string is valid url
// --------------------------------
// RETURN: <string> input url (with added "http://" if needed) or <false> if check failed
// --------------------------------
    urlParts = url.split("?").shift().split("://"); // No queries
    if(urlParts.length == 1) { urlParts.unshift("http") };
    characterTest = urlParts[1].replace(/[^a-z0-9|^\-|^_|^.|^~|^\/]/gi, "");
    if(
        urlParts[0].length >= 3 &&
        urlParts[1] == characterTest &&
        urlParts[1].indexOf("//") < 0 &&
        urlParts[1].indexOf("..") < 0 &&
        urlParts[1].indexOf(".") > -1
    ) {
        return url;
    }
    else {
        return false;
    };
};

// ==========================================
//                  String
// ==========================================

String.prototype.killPL = function() {
// -------------------------------------------
// this = <string>
// -------------------------------------------
// RETURNS: <string> With non breaking spaces (HTML)
// -------------------------------------------
    var text = this;
    var chars_map = {
        "ą": "a", "ć": "c", "ę": "e", "ł": "l", "ń": "n", "ó": "o", "ś": "s", "ż": "z", "ź": "z",
        "Ą": "A", "Ć": "C", "Ę": "E", "Ł": "L", "Ń": "N", "Ó": "O", "Ś": "S", "Ż": "Z", "Ź": "Z"
    }
    for(i in chars_map) {
        var re = new RegExp(i,"g");
        text = text.replace(re, chars_map[i])

    }
    return text;
};

String.prototype.makeId = function() {
// -------------------------------------------
// this = <string>
// -------------------------------------------
// RETURNS: <string> Simplified to math standards of filename / html tag etc.
// -------------------------------------------
    var text = this;
    text = text.killPL().toLowerCase();
    text = text.replace(/ |\n|\t/g, "_");
    text = text.replace(/[^a-z0-9|^_|^\-|^+|^(|^)]/gi, "");
    text = text.replace(/_+/g, "_");
    return text;
};

String.prototype.nbsp = function() {
// -------------------------------------------
// this = <string>
// -------------------------------------------
// RETURNS: <string> With non breaking spaces (HTML)
// -------------------------------------------
    return this.replace(/ /g, "&nbsp;");
};

String.prototype.capitalize = function(mode) { // used in: translate()
// -------------------------------------------
// this = <string>
// mode = <string> method MODE
//     "string", undefined -> eg: "Lorem ipsum. lorem ipsum." (default)
//     "sentence", "paragraph" -> eg: "Lorem ipsum. Lorem ipsum."
//     "title", "word" -> eg: "Lorem Ipsum. Lorem Ipsum."
// -------------------------------------------
// RETURNS: <string> Capitalized input string
// -------------------------------------------
    if(mode == "sentence" || mode == "paragraph") {
        var sentences = this.split(". ");
        for(n = 0; n < sentences.length; n++) { sentences[n] = sentences[n].capitalize(); };
        return sentences.join(". ");
    }
    else if(mode == "title" || mode == "word") {
        var words = this.split(" ");
        for(n = 0; n < words.length; n++) { words[n] = words[n].capitalize(); };
        return words.join(" ");
    }
    else { // "string"
        return this.substr(0, 1).toUpperCase() + this.substr(1).toLowerCase();
    };
};

String.prototype.path = function(mode) { // used in: *common*
// -------------------------------------------
// this = <string> file path
// mode = <string> method MODE
//     undefined -> "folder/file.txt"
//     "basename"-> eg: "file" or last folder in specified path
//     "dirname" -> eg: "folder"
//     "extension" -> eg: "txt"
//     "filename" -> eg: "file.txt"
// -------------------------------------------
// RETURNS: <string> Specified path data part
// -------------------------------------------
    // Path analyze
    var dirname = this.split("/");

    if(dirname.length > 1) { // folder
        var basename = dirname.pop();
        dirname = dirname.join("/");
    }
    else { // no folder
        var basename = dirname.shift();
        dirname = "";
    };
    // File analyze
    var filename = basename.split(".");
    if(filename.length > 1) { // extension
        var extension = filename.pop();
        filename = filename.join(".");
    }
    else { // no extension -> no file
        basename = filename.shift();
        if(dirname == "") { dirname = basename; }
        else { dirname = dirname + "/" + basename; };
        filename = "";
        var extension = "";
    };
    // OUTPUT
    if(mode == "basename") { return basename; }
    else if(mode == "dirname") { return dirname; }
    else if(mode == "extension") { return extension; }
    else if(mode == "filename") { return filename; }
    else { return this; };
};

// ==========================================
//                  BBCode
// ==========================================
  
function BBCode(text, site_root_path) {
// --------------------------------
// text = <string>
// --------------------------------
// RETURN: <string> Input text with BBCode applied
// --------------------------------
    if(typeof(site_root_path) != "string") { site_root_path = ""; };
    
    text = text.replace( /\n/ig, "<br>"); // line break(s)
    text = text.replace( /\\\[/g, "&#91;"); // "["
    text = text.replace( /\\\]/g, "&#93;"); // "]"

    // Banner
    text = text.replace( /\[banner(.*?)\[img=(.*?)\](.*?)\[\/banner\]/ig, "[banner$1<div class='figure' style='background-image:url(\"" + site_root_path + "/$2\")'></div><p>$3</p>[/banner]" ); // banner image
    text = text.replace( /\[\/banner\]<br>/ig, "[/banner]"); // enter fix 1
    text = text.replace( /<br>\[banner(.*?)\]/ig, "[banner$1]"); // enter fix 2
    text = text.replace( /\[banner\|r\](.*?)\[\/banner\]/ig, "</p><div class='banner text_right'>$1</div><p>"); // banner|r
    text = text.replace( /\[banner\|l\](.*?)\[\/banner\]/ig, "</p><div class='banner text_left'>$1</div><p>"); // banner|l
    text = text.replace( /\[banner\](.*?)\[\/banner\]/ig, "</p><div class='banner'>$1</div><p>"); // banner
    // Button
    text = text.replace( /\[\/button\]\[button(.*?)\]/ig, ""); // join touching buttons (unify size -> first)
    text = text.replace( /\[\/button\]<br>/ig, "[/button]"); // enter fix 1
    text = text.replace( /<br>\[button(.*?)\]/ig, "[button$1]"); // enter fix 2
    text = text.replace( /\[button\|(.*?)\](.*?)\[\/button\]/ig, "</p><div class='button size_$1'>$2</div><p>"); // sized button
    text = text.replace( /\[button\](.*?)\[\/button\]/ig, "</p><div class='button'>$1</div><p>"); // button

    text = text.replace( /\[img=(.*?)\]/ig, "<img src='" + site_root_path + "/$1'>"); // image

    text = text.replace( /\[hr\]<br>/ig, "[hr]"); // enter fix

    // ====== Main style ======
    // Forced link types
    text = text.replace( /\[url\|e=(.*?)\](.*?)\[\/url\]/ig, "<a href='$1' target='_blank'>$2</a>"); // external links
    text = text.replace( /\[url\|i=(.*?)\](.*?)\[\/url\]/ig, "<a href='$1'>$2</a>"); // internal links
    // Complex
    text = text.replace( /\[url=(.*?)\](.*?)\[\/url\]/ig, "<a href='$1' target='_blank'>$2</a>"); // links
    text = text.replace( /\[email=(.*?)\](.*?)\[\/email\]/ig, "<a href=\"mailto:$1\">$2</a>"); // mailto
    text = text.replace( /\[tel=(.*?)\](.*?)\[\/tel\]/ig, "<a href=\"tel:$1\">$2</a>"); // tel
    text = text.replace( /\[color=(.*?)\](.*?)\[\/color\]/ig, "<span class='color_$1'>$2</span>"); // color
    text = text.replace( /\[size=(.*?)\](.*?)\[\/size\]/ig, "<span class='size_$1'>$2</span>"); // size
    text = text.replace( /\[\hr\]/ig, "</p><div class='hr'><hr></div><p>"); // horizintal line
    // Simple
    text = text.replace( /\[\*\]/ig, "&bull;&nbsp;&nbsp;"); // bullet character
    text = text.replace( /\[tab\]/ig, "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;"); // tab character
    text = text.replace( /\[b\](.*?)\[\/b\]/ig, "<b>$1</b>"); // bold
    text = text.replace( /\[i\](.*?)\[\/i\]/ig, "<i>$1</i>"); // italic
    text = text.replace( /\[u\](.*?)\[\/u\]/ig, "<u>$1</u>"); // italic
    text = text.replace( /\[sup\](.*?)\[\/sup\]/ig, "<sup>$1</sup>"); // superscript
    text = text.replace( /\[sub\](.*?)\[\/sub\]/ig, "<sub>$1</sub>"); // subscript
    text = text.replace( /\[center\](.*?)\[\/center\]/ig, "</p><p style='text-align:center'>$1</p><p>"); // center open

    text = text.replace( / +/g, " "); // multiple spaces
    text = text.replace( / (.?) /g, " $1&nbsp;"); // widow letters

    // List
    if(lists = text.match(/\[list\](.*?)\[\/list\]/ig)) {
        for(i in lists) {
            list = lists[i];
            list = list.replace(/\[list\](.*?)\[\/list\]/ig, "$1");
            list = "</p><ul class='preview'><li>" + list.replace( /<br>/ig, "</li><li>" ) + "</li></ul><p>";
            list = list.replace(/<li>-/ig, "<li style='list-style:none'>-"); // sublist point
            list = list.replace(/<li><\/li>/ig, "<li style='list-style:none'>&nbsp;</li>"); // empty line -> no list bullet
            text = text.replace(lists[i], list);
        };
    };

    return text;
};

function noBBCode(text) {
// --------------------------------
// text = <string>
// --------------------------------
// RETURN: <string> Input text stripped of any BBCode tags
// --------------------------------
    text = text.replace( /\n/ig, "<br>"); // line break(s)

    text = text.replace(/\[url\|(.*?)\](.*?)\[\/url\]/ig, "$2"); // link with option
    text = text.replace(/\[url=(.*?)\](.*?)\[\/url\]/ig, "$2");	// link
    text = text.replace(/\[email=(.*?)\](.*?)\[\/email\]/ig, "$2"); // mailto link
    text = text.replace(/\[tel=(.*?)\](.*?)\[\/tel\]/ig, "$2"); // tel link
    text = text.replace(/\[center\](.*?)\[\/center\]/ig, "$1"); // center
    text = text.replace(/\[list\](.*?)\[\/list\]/ig, "$1"); // list
    text = text.replace(/\[(.?)\]/ig, ""); // simple tag open
    text = text.replace(/\[\/(.?)\]/ig, ""); // simpl tag close
    text = text.replace(/\[\*\]/ig, ""); // bullet
    text = text.replace(/\[tab\]/ig, ""); // bullet
    text = text.replace(/\[\hr\]/ig, ""); // horizontal line

    // Attachements - photo/file
    text = text.replace(/\[banner(.*?)\](.*?)\[\/banner\]/ig, "$2");
    text = text.replace(/\[button(.*?)\](.*?)\[\/button\]/ig, "$2");
    text = text.replace(/\[img=(.*?)\]/ig, "");

    text = text.replace( /<br>/ig, "\n"); // line break(s)
    return text;
};

// ==========================================
//               Miscleanous
// ==========================================

function autoUrl(href) {
// -------------------------------------------
// href = <string>
// -------------------------------------------
// RETURN: <string> Input HREF (URL) with encoded queries part
// -------------------------------------------
    href = href.split("?");
    if(href.length > 1) { href = href.shift() + "?" + encodeURIComponent(href.join("?")); }
    else { href = href.shift(); };
    return href;
};

function currentTime(format) { // used in: initializeNewPost()
// -------------------------------------------
// format = <string> output format, eg: "yyyy-dd-mm" -> "2016-02-11"
// -------------------------------------------
// RETURN: <string> time data in specified format
// -------------------------------------------
    currentdate = new Date();
    time = {
        yyyy: currentdate.getFullYear(),
        mm: digits(currentdate.getMonth()+1, 2),
        dd: digits(currentdate.getDate(), 2),
        ho: digits(currentdate.getHours(), 2),
        mi: digits(currentdate.getMinutes(), 2),
        se: digits(currentdate.getSeconds(), 2)
    };
    for(key in time) {
        format = format.replace(key, time[key]);
    };
    return format;
};

function dateToString(format, date) {
// -------------------------------------------
// format = <string> output format
// date = <object> time/date object, if not specified -> currend date/time will be used
// -------------------------------------------
// RETURN: <string> time data in specified format
// -------------------------------------------
    if(jQuery.type(date) != "date") { date = new Date(); }
    iso_weekday = [ 7, 1, 2, 3, 4, 5, 6 ];
    var keys = {
        yyyy: date.getFullYear(),                   // Year, full
        dd: date.getDate().toDigits(2),             // Day, 2 digits
        D: date.getDate(),                          // Day
        mm: (date.getMonth() + 1).toDigits(2),      // Month, 2 digits
        M: (date.getMonth() + 1),                   // Month
        ho: date.getHours().toDigits(2),            // Hours, 2 digits
        mi: date.getMinutes().toDigits(2),          // Hours, 2 digits
        se: date.getSeconds().toDigits(2),          // Hours, 2 digits
        W: iso_weekday[ date.getDay() ],            // ISO-8601 weekday number (1=Monday, 7=Sunday)
        w: date.getDay(),                           // Weekday number (0=Sunday, 6=Saturday)
    }
    for(i in keys) {
        key = new RegExp(i, "g");
        format = format.replace(key, keys[i]);
    };
    return format;
};

function downloadFile(href) {
// -------------------------------------------
// href = <string> file link
// -------------------------------------------
// Save file to client's disk
// -------------------------------------------
    var element = document.createElement('a');
    element.setAttribute('href', href);
    element.setAttribute('download', href.path("basename"));
    element.style.display = 'none';
    document.body.appendChild(element);
    element.click();
    document.body.removeChild(element);
}

function getMonthDays(date, option) {
// -------------------------------------------
// date = <object> Date object
// option = <string> "week": from Monday to Sunday containing all month days (optional)
// -------------------------------------------
// RETURNS: <array> Month days objects list + optional prefix/suffix days to complete full weeks table
// -------------------------------------------
    var nextMonth = new Date(date.getFullYear(), date.getMonth() + 1, 1);
    var nextDay = new Date(date.getFullYear(), date.getMonth(), 1);
    // ====== Options ======
    if(option == "week") {
        iso_weekday = [ 7, 1, 2, 3, 4, 5, 6 ];
        // Add month prefix
        diff = iso_weekday[nextDay.getDay()] - 1;
        if(diff > 0) {
            nextDay.setDate(nextDay.getDate() - diff);
        };
        // Add month suffix
        diff = 8 - iso_weekday[nextMonth.getDay()];
        //alert(diff);
        if(diff > 0) {
            nextMonth.setDate(nextMonth.getDate() + diff);
        }
    }
    // ====== Get days loop ======
    monthDays = [];
    while(nextDay < nextMonth)  {
        //alert(dateToString("yyyy-mm-dd (W)", nextDay))
        monthDays[monthDays.length] = new Date(nextDay);
        nextDay.setDate(nextDay.getDate() + 1);
    }
    return monthDays;
};

function textareaResize($textarea, min_height) {
// -------------------------------------------
// $textarea = <element> textarea jquery element
// min_height = <integer> min-height in [px]
// -------------------------------------------
// Automatic textarea height resize
// --------------------------------
    $textarea.height(min_height);
    if($textarea[0].scrollHeight > min_height) {
        $textarea.height($textarea[0].scrollHeight);
    };
};

$.fn.selectRange = function(start, end) {
// -------------------------------------------
// start = <integer> selection start position
// end = <integer> selection end position (optional)
// -------------------------------------------
// Set cursor/selection position in textarea
// -------------------------------------------
    if(end == undefined) {
        end = start;
    }
    return this.each(function() {
        if('selectionStart' in this) {
            this.selectionStart = start;
            this.selectionEnd = end;
        } else if(this.setSelectionRange) {
            this.setSelectionRange(start, end);
        } else if(this.createTextRange) {
            var range = this.createTextRange();
            range.collapse(true);
            range.moveEnd('character', end);
            range.moveStart('character', start);
            range.select();
        }
    });
};

function uniqId(outputLength = 13) {
// ------------------------
// outputLength = <integer> Output string length
// ------------------------
// RETURNS: <string> Random hex digits string
// ------------------------
    var output = "";
    for(i = 0; i < outputLength; i++) {
        chr = Math.floor(Math.random() * 16);
        if(chr >= 10) { chr = String.fromCharCode(97 + chr - 10); }
        output = output + chr
    }
    return output;
}

function urlQuery(actions) {
// -------------------------------------------
// actions = <string> for query modifiy, or <false> to delete all,
//              options (multiple divided with comas):
//              "key=value": Replace query
//              "+key=value": Add (or modify) query
//              "-key": Delete query
//              "key": Read query (no modification)
// -------------------------------------------
// MODIFIES: current URL (without reloading a page) / or RETURNS: <string> Query value(s) in read mode separated with comas
// -------------------------------------------
    // ====== Output variables ======
    var read = [];
    var queries = {};
    // ====== Get all gata ======
    var original_url = location.href; // Original url HREF
    var url = original_url.split("?");
    var url_base = url[0];
    if(url.length == 2) { url = url[1].split("&"); } else { url = []; };
    for(i in url) {
        query = url[i].split("=");
        if(query.length == 2) {
            queries[ query[0] ] = query[1];
        };
    };
    // ====== Actions ======
    if(typeof(actions) != "string") {
        queries = {}; // Delete all
    }
    else if(actions.length) {
        actions = actions.split(",");
        for(i in actions) {
            action = actions[i];
            key_val = action.split("=");
            // +key=value -> add/mod value
            if(action.substr(0, 1) == "+" && key_val.length == 2) {
                queries[ key_val[0].substr(1) ] = key_val[1];
            }
            // key=value -> mod existing value
            else if(key_val.length == 2 && queries[ key_val[0] ]) {
                queries[ key_val[0] ] = key_val[1];
            }
            // -key -> delete
            else if(action.substr(0, 1) == "-") {
                delete queries[ action.substr(1) ];
            }
            // key -> READ value
            else if(action.length > 0 && queries[ action ]) {
                read[read.length] = queries[ action ];
            };
        }
    };
    // ====== Build new URL ======
    var url = [];
    for(key in queries) {
        value = queries[key];
        url[url.length] = key + "=" + value;
    };
    if(url.length) {
        updated_url = url_base + "?" + url.join("&");
    }
    else {
        updated_url = url_base;
    }
    // ====== Output ======
    // Modify URL
    if(updated_url != original_url) {
        //alert("updated URL:\n" + updated_url);
        window.history.pushState(window.location.href, document.title, updated_url);
    }
    else {
        //alert("URL not changed");
    }
    // Return value(s)
    if(read.length) {
        //alert("read VALUES:\n" + read);
        return read.join(",");
    };
};
